@@ -1,15 +1,14 @@
 use clap::Parser;
-use colorgrad::{preset, Color, Gradient};
-use std::io::{self, BufReader, IsTerminal, Write};
+use colorgrad::{Color, GimpGradient, Gradient};
+use std::io::{self, BufReader, IsTerminal, Read, Write};
 use std::{ffi::OsStr, fs::File, process::exit};
 
 mod cli;
 use cli::{BlendMode, Interpolation, Opt, OutputColor, PRESET_NAMES};
 
 mod svg_gradient;
-use svg_gradient::parse_svg;
-
 mod util;
+use util::bold;
 
 #[derive(PartialEq)]
 enum OutputMode {
@@ -18,9 +17,9 @@ enum OutputMode {
     ColorsSample,
 }
 
-struct GradientApp {
+struct GradientApp<'a> {
     opt: Opt,
-    stdout: io::Stdout,
+    stdout: io::StdoutLock<'a>,
     is_terminal: bool,
     output_mode: OutputMode,
     output_format: OutputColor,
@@ -32,7 +31,7 @@ struct GradientApp {
     height: usize,
 }
 
-impl GradientApp {
+impl GradientApp<'_> {
     fn new(opt: Opt, stdout: io::Stdout) -> Self {
         let term_width = if let Some((terminal_size::Width(w), _)) = terminal_size::terminal_size()
         {
@@ -74,7 +73,7 @@ impl GradientApp {
 
         Self {
             output_mode,
-            stdout,
+            stdout: stdout.lock(),
             is_terminal,
             use_solid_bg: opt.background.is_some(),
             background,
@@ -92,9 +91,9 @@ impl GradientApp {
             self.width = self.term_width.min(80);
             self.height = 2;
 
-            for name in &PRESET_NAMES {
+            for name in PRESET_NAMES {
                 writeln!(self.stdout, "{name}")?;
-                self.opt.preset = Some(name.to_string());
+                self.opt.preset = Some(name.into());
                 self.preset_gradient()?;
             }
 
@@ -102,7 +101,10 @@ impl GradientApp {
         }
 
         if self.opt.named_colors {
-            for (&name, &[r, g, b]) in csscolorparser::NAMED_COLORS.entries() {
+            let mut colors: Vec<_> = csscolorparser::NAMED_COLORS.entries().collect();
+            colors.sort_by(|a, b| a.0.cmp(b.0));
+
+            for (&name, &[r, g, b]) in &colors {
                 writeln!(
                     self.stdout,
                     "\x1B[48;2;{r};{g};{b}m   \x1B[49;38;2;{r};{g};{b}m #{r:02x}{g:02x}{b:02x}\x1B[39m {name}"
@@ -129,60 +131,39 @@ impl GradientApp {
     }
 
     fn preset_gradient(&mut self) -> io::Result<i32> {
-        let grad: Box<dyn Gradient> = match self
-            .opt
-            .preset
-            .as_ref()
-            .unwrap()
-            .to_lowercase()
-            .replace('-', "_")
-            .as_ref()
-        {
-            "blues" => Box::new(preset::blues()),
-            "br_bg" => Box::new(preset::br_bg()),
-            "bu_gn" => Box::new(preset::bu_gn()),
-            "bu_pu" => Box::new(preset::bu_pu()),
-            "cividis" => Box::new(preset::cividis()),
-            "cool" => Box::new(preset::cool()),
-            "cubehelix" => Box::new(preset::cubehelix_default()),
-            "gn_bu" => Box::new(preset::gn_bu()),
-            "greens" => Box::new(preset::greens()),
-            "greys" => Box::new(preset::greys()),
-            "inferno" => Box::new(preset::inferno()),
-            "magma" => Box::new(preset::magma()),
-            "or_rd" => Box::new(preset::or_rd()),
-            "oranges" => Box::new(preset::oranges()),
-            "pi_yg" => Box::new(preset::pi_yg()),
-            "plasma" => Box::new(preset::plasma()),
-            "pr_gn" => Box::new(preset::pr_gn()),
-            "pu_bu" => Box::new(preset::pu_bu()),
-            "pu_bu_gn" => Box::new(preset::pu_bu_gn()),
-            "pu_or" => Box::new(preset::pu_or()),
-            "pu_rd" => Box::new(preset::pu_rd()),
-            "purples" => Box::new(preset::purples()),
-            "rainbow" => Box::new(preset::rainbow()),
-            "rd_bu" => Box::new(preset::rd_bu()),
-            "rd_gy" => Box::new(preset::rd_gy()),
-            "rd_pu" => Box::new(preset::rd_pu()),
-            "rd_yl_bu" => Box::new(preset::rd_yl_bu()),
-            "rd_yl_gn" => Box::new(preset::rd_yl_gn()),
-            "reds" => Box::new(preset::reds()),
-            "sinebow" => Box::new(preset::sinebow()),
-            "spectral" => Box::new(preset::spectral()),
-            "turbo" => Box::new(preset::turbo()),
-            "viridis" => Box::new(preset::viridis()),
-            "warm" => Box::new(preset::warm()),
-            "yl_gn" => Box::new(preset::yl_gn()),
-            "yl_gn_bu" => Box::new(preset::yl_gn_bu()),
-            "yl_or_br" => Box::new(preset::yl_or_br()),
-            "yl_or_rd" => Box::new(preset::yl_or_rd()),
-            _ => {
-                writeln!(io::stderr(), "Error: Invalid preset gradient name. Use -l flag to list all preset gradient names.")?;
-                return Ok(1);
+        use colorgrad::preset::*;
+
+        fn cubehelix() -> CubehelixGradient {
+            cubehelix_default()
+        }
+
+        macro_rules! presets {
+            ($($name:ident),*) => {
+                match self
+                .opt
+                .preset
+                .as_ref()
+                .unwrap()
+                .to_lowercase()
+                .replace('-', "_")
+                .as_ref()
+                {
+                    $(stringify!($name) => self.handle_output(&$name())?,)*
+                    _ => {
+                        writeln!(io::stderr(), "Error: Invalid preset gradient name. Use -l flag to list all preset gradient names.")?;
+                        return Ok(1);
+                    }
+                }
             }
-        };
+        }
+
+        presets!(
+            blues, br_bg, bu_gn, bu_pu, cividis, cool, cubehelix, gn_bu, greens, greys, inferno,
+            magma, or_rd, oranges, pi_yg, plasma, pr_gn, pu_bu, pu_bu_gn, pu_or, pu_rd, purples,
+            rainbow, rd_bu, rd_gy, rd_pu, rd_yl_bu, rd_yl_gn, reds, sinebow, spectral, turbo,
+            viridis, warm, yl_gn, yl_gn_bu, yl_or_br, yl_or_rd
+        );
 
-        self.handle_output(grad)?;
         Ok(0)
     }
 
@@ -206,23 +187,23 @@ impl GradientApp {
             _ => colorgrad::BlendMode::Oklab,
         });
 
-        let grad: Box<dyn Gradient> = match self.opt.interpolation {
+        match self.opt.interpolation {
             Some(Interpolation::Linear) => match gb.build::<colorgrad::LinearGradient>() {
-                Ok(g) => Box::new(g),
+                Ok(g) => self.handle_output(&g)?,
                 Err(e) => {
                     writeln!(io::stderr(), "Custom gradient error: {e}")?;
                     return Ok(1);
                 }
             },
             Some(Interpolation::Basis) => match gb.build::<colorgrad::BasisGradient>() {
-                Ok(g) => Box::new(g),
+                Ok(g) => self.handle_output(&g)?,
                 Err(e) => {
                     writeln!(io::stderr(), "Custom gradient error: {e}")?;
                     return Ok(1);
                 }
             },
             _ => match gb.build::<colorgrad::CatmullRomGradient>() {
-                Ok(g) => Box::new(g),
+                Ok(g) => self.handle_output(&g)?,
                 Err(e) => {
                     writeln!(io::stderr(), "Custom gradient error: {e}")?;
                     return Ok(1);
@@ -230,11 +211,12 @@ impl GradientApp {
             },
         };
 
-        self.handle_output(grad)?;
         Ok(0)
     }
 
     fn file_gradient(&mut self) -> io::Result<i32> {
+        use colorgrad::{BasisGradient, CatmullRomGradient, LinearGradient};
+
         let ggr_bg_color = if let Some(ref c) = self.opt.ggr_bg {
             c.clone()
         } else {
@@ -247,96 +229,114 @@ impl GradientApp {
             Color::new(0.0, 0.0, 0.0, 1.0)
         };
 
+        let show_info = self.is_terminal || self.output_mode == OutputMode::Gradient;
         let mut status = 0;
 
         for path in self.opt.file.as_ref().unwrap().clone() {
             if !path.exists() {
-                write!(
-                    self.stdout,
-                    "{}\n  \x1B[31mFile not found\x1B[39m\n",
-                    &path.display()
-                )?;
+                eprintln!("{}: file not found.", &path.display());
                 status = 1;
                 continue;
             }
 
-            if let Some(ext) = path.extension().and_then(OsStr::to_str) {
-                match ext.to_lowercase().as_ref() {
-                    "ggr" => {
-                        if self.is_terminal || (self.output_mode == OutputMode::Gradient) {
-                            write!(self.stdout, "{}", &path.display())?;
-                        }
+            let Some(ext) = path.extension().and_then(OsStr::to_str) else {
+                eprintln!("{}: file format not supported.", &path.display());
+                status = 1;
+                continue;
+            };
 
-                        let f = File::open(&path)?;
+            let ext = ext.to_lowercase();
 
-                        match colorgrad::GimpGradient::new(
-                            BufReader::new(f),
-                            &ggr_fg_color,
-                            &ggr_bg_color,
-                        ) {
-                            Ok(grad) => {
-                                if self.is_terminal || (self.output_mode == OutputMode::Gradient) {
-                                    writeln!(self.stdout, " \x1B[1m{}\x1B[0m", grad.name())?;
-                                }
-
-                                self.handle_output(Box::new(grad))?;
-                            }
-
-                            Err(err) => {
-                                if self.is_terminal || (self.output_mode == OutputMode::Gradient) {
-                                    writeln!(self.stdout, "\n  \x1B[31m{err}\x1B[39m")?;
-                                }
-                            }
+            if &ext == "ggr" {
+                match GimpGradient::new(
+                    BufReader::new(File::open(&path)?),
+                    &ggr_fg_color,
+                    &ggr_bg_color,
+                ) {
+                    Ok(grad) => {
+                        if show_info {
+                            writeln!(self.stdout, "{} {}", &path.display(), bold(grad.name()))?;
                         }
+                        self.handle_output(&grad)?;
                     }
-
-                    "svg" => {
-                        let filename = &path.display().to_string();
-                        let gradients =
-                            parse_svg(path.into_os_string().into_string().unwrap().as_ref());
-
-                        if (self.is_terminal || (self.output_mode == OutputMode::Gradient))
-                            && gradients.is_empty()
-                        {
-                            writeln!(self.stdout, "{filename}")?;
-                            writeln!(self.stdout, "  \x1B[31mNo gradients.\x1B[39m")?;
-                        }
-
-                        for (grad, id) in gradients {
-                            let (id, stop) = if let Some(id) = id {
-                                if let Some(ref id2) = self.opt.svg_id {
-                                    if &id == id2 {
-                                        (format!("#{id}"), true)
-                                    } else {
-                                        continue;
-                                    }
-                                } else {
-                                    (format!("#{id}"), false)
-                                }
-                            } else {
-                                ("".to_string(), false)
-                            };
-
-                            if self.is_terminal || (self.output_mode == OutputMode::Gradient) {
-                                writeln!(self.stdout, "{filename} \x1B[1m{id}\x1B[0m")?;
-                            }
-
-                            self.handle_output(Box::new(grad))?;
-
-                            if stop {
-                                break;
-                            }
-                        }
+                    Err(_) => {
+                        eprintln!("{} (invalid GIMP gradient)", &path.display());
+                        status = 1;
+                        continue;
                     }
-                    _ => continue,
                 }
+            } else if &ext == "svg" {
+                let mut file = File::open(&path)?;
+                let mut content = String::new();
+                file.read_to_string(&mut content)?;
+                let svg_grads = svg_gradient::parse_svg(&content, self.opt.svg_id.as_deref());
+
+                let cmode = match self.opt.blend_mode {
+                    Some(BlendMode::Rgb) => colorgrad::BlendMode::Rgb,
+                    Some(BlendMode::LinearRgb) => colorgrad::BlendMode::LinearRgb,
+                    Some(BlendMode::Lab) => colorgrad::BlendMode::Lab,
+                    _ => colorgrad::BlendMode::Oklab,
+                };
+                let mut valid = 0;
+                let mut invalid = 0;
+
+                for mut sg in svg_grads {
+                    assert_eq!(sg.colors.len(), sg.pos.len());
+
+                    let id = sg
+                        .id
+                        .as_ref()
+                        .map(|s| format!("#{s}"))
+                        .unwrap_or("[without id]".into());
+
+                    let Some(mut gb) = sg.gradient_builder() else {
+                        eprintln!("{} {} (invalid gradient)", &path.display(), bold(&id));
+                        status = 1;
+                        invalid += 1;
+                        continue;
+                    };
+
+                    if show_info {
+                        writeln!(self.stdout, "{} {}", &path.display(), bold(&id))?;
+                    }
+
+                    gb.mode(cmode);
+
+                    match self.opt.interpolation {
+                        Some(Interpolation::Linear) => {
+                            let g: LinearGradient = gb.build().unwrap();
+                            self.handle_output(&g)?;
+                        }
+                        Some(Interpolation::Basis) => {
+                            let g: BasisGradient = gb.build().unwrap();
+                            self.handle_output(&g)?;
+                        }
+                        _ => {
+                            let g: CatmullRomGradient = gb.build().unwrap();
+                            self.handle_output(&g)?;
+                        }
+                    }
+                    valid += 1;
+                }
+
+                if valid == 0 && invalid == 0 {
+                    if self.opt.svg_id.is_some() {
+                        eprintln!("{} -- (nothing matched)", &path.display(),);
+                    } else {
+                        eprintln!("{} -- (no gradients found)", &path.display());
+                    }
+                    status = 1;
+                }
+            } else {
+                eprintln!("{}: file format not supported.", &path.display());
+                status = 1;
             }
         }
 
         Ok(status)
     }
 
-    fn handle_output(&mut self, grad: Box<dyn Gradient>) -> io::Result<i32> {
+    fn handle_output(&mut self, grad: &dyn Gradient) -> io::Result<i32> {
         match self.output_mode {
             OutputMode::Gradient => self.display_gradient(grad),
 
@@ -351,65 +351,60 @@ impl GradientApp {
             }
 
             OutputMode::ColorsSample => {
-                let mut colors = Vec::with_capacity(self.opt.sample.as_ref().unwrap().len());
-                for pos in self.opt.sample.as_ref().unwrap().iter() {
-                    let mut col = grad.at(*pos).clamp();
-                    if self.use_solid_bg {
-                        util::blend_on(&mut col, &self.background);
-                    }
-                    colors.push(col);
-                }
+                let colors: Vec<_> = self
+                    .opt
+                    .sample
+                    .as_ref()
+                    .unwrap()
+                    .iter()
+                    .map(|t| {
+                        let mut c = grad.at(*t).clamp();
+                        if self.use_solid_bg {
+                            util::blend_on(&mut c, &self.background);
+                        }
+                        c
+                    })
+                    .collect();
                 self.display_colors(&colors)
             }
         }
     }
 
-    fn display_gradient(&mut self, grad: Box<dyn Gradient>) -> io::Result<i32> {
-        let (dmin, dmax) = grad.domain();
-        let w2 = (self.width * 2 - 1) as f32;
-        let [cb_0, cb_1] = &self.cb_color;
+    fn display_gradient(&mut self, grad: &dyn Gradient) -> io::Result<i32> {
+        let colors = grad.colors(self.width * 2);
+        let mut out = io::BufWriter::new(&mut self.stdout);
 
         for y in 0..self.height {
-            let mut i = 0;
-
-            for x in 0..self.width {
+            for (x, cols) in colors.chunks_exact(2).enumerate() {
                 let bg_color = if self.use_solid_bg {
                     &self.background
                 } else if ((x / 2) & 1) ^ (y & 1) == 0 {
-                    cb_0
+                    &self.cb_color[0]
                 } else {
-                    cb_1
+                    &self.cb_color[1]
                 };
 
-                let col_l = grad.at(util::remap(i as f32, 0.0, w2, dmin, dmax));
-                i += 1;
+                let [a, b, c, _] = util::blend_color(&cols[0], bg_color).to_rgba8();
+                let [d, e, f, _] = util::blend_color(&cols[1], bg_color).to_rgba8();
 
-                let col_r = grad.at(util::remap(i as f32, 0.0, w2, dmin, dmax));
-                i += 1;
-
-                let col_l = util::blend_color(&col_l, bg_color).to_rgba8();
-                let col_r = util::blend_color(&col_r, bg_color).to_rgba8();
-
-                write!(
-                    self.stdout,
-                    "\x1B[38;2;{};{};{};48;2;{};{};{}m\u{258C}",
-                    col_l[0], col_l[1], col_l[2], col_r[0], col_r[1], col_r[2]
-                )?;
+                write!(out, "\x1B[38;2;{a};{b};{c};48;2;{d};{e};{f}m\u{258C}",)?;
             }
 
-            writeln!(self.stdout, "\x1B[39;49m")?;
+            writeln!(out, "\x1B[39;49m")?;
         }
 
+        out.flush()?;
         Ok(0)
     }
 
     fn display_colors(&mut self, colors: &[Color]) -> io::Result<i32> {
+        let mut out = io::BufWriter::new(&mut self.stdout);
+
         if self.opt.array {
-            let mut cols = Vec::with_capacity(colors.len());
-            for col in colors {
-                cols.push(util::format_color(col, self.output_format));
-            }
-            writeln!(self.stdout, "{cols:?}")?;
+            let f = self.output_format;
+            let cols: Vec<_> = colors.iter().map(|c| util::format_color(c, f)).collect();
+            writeln!(out, "{cols:?}")?;
+            out.flush()?;
             return Ok(0);
         }
 
@@ -417,12 +412,13 @@ impl GradientApp {
             if self.output_format != OutputColor::Hex {
                 for col in colors {
                     writeln!(
-                        self.stdout,
+                        out,
                         "{} {}",
-                        util::fmt_color(col, &self.cb_color, 7),
+                        util::color_to_ansi(col, &self.cb_color, 7),
                         util::format_color(col, self.output_format)
                     )?;
                 }
+                out.flush()?;
                 return Ok(0);
             }
 
@@ -434,7 +430,7 @@ impl GradientApp {
             for (i, col) in colors.iter().enumerate() {
                 let hex = util::format_color(col, self.output_format);
                 let wc = hex.len();
-                buff0.push_str(&util::fmt_color(col, &self.cb_color, wc));
+                buff0.push_str(&util::color_to_ansi(col, &self.cb_color, wc));
                 buff1.push_str(&hex);
                 w += wc;
                 if w < self.term_width {
@@ -449,22 +445,20 @@ impl GradientApp {
                 };
 
                 if w + nwc > self.term_width || i == last {
-                    writeln!(self.stdout, "{}\n{}", buff0, buff1)?;
+                    writeln!(out, "{}\n{}", buff0, buff1)?;
                     buff0.clear();
                     buff1.clear();
                     w = 0;
                 }
             }
+            out.flush()?;
             return Ok(0);
         }
 
         for col in colors {
-            writeln!(
-                self.stdout,
-                "{}",
-                util::format_color(col, self.output_format)
-            )?;
+            writeln!(out, "{}", util::format_color(col, self.output_format))?;
         }
+        out.flush()?;
         Ok(0)
     }
 
@@ -476,10 +470,6 @@ impl GradientApp {
                 .collect()
         }
 
-        fn bold(s: &str) -> String {
-            format!("\x1B[1m{s}\x1B[0m")
-        }
-
         let prompt = "\u{21AA} ";
         self.width = self.term_width.min(80);
         self.height = 2;
@@ -491,7 +481,7 @@ impl GradientApp {
         )?;
         writeln!(self.stdout, "{}", bold("EXAMPLES:"))?;
         writeln!(self.stdout, "{prompt} gradient --preset rainbow")?;
-        self.opt.preset = Some("rainbow".to_string());
+        self.opt.preset = Some("rainbow".into());
         self.preset_gradient()?;
 
         writeln!(
@@ -509,7 +499,7 @@ impl GradientApp {
 
         writeln!(self.stdout, "{prompt} gradient --css 'white, 25%, blue'")?;
         self.opt.custom = None;
-        self.opt.css = Some("white, 25%, blue".to_string());
+        self.opt.css = Some("white, 25%, blue".into());
         self.custom_gradient()?;
 
         writeln!(
@@ -524,14 +514,13 @@ impl GradientApp {
         writeln!(self.stdout, "Neon_Green.ggr {}", bold("Neon Green"))?;
         const GGR_SAMPLE: &str = include_str!("../data/Neon_Green.ggr");
         let color = Color::default();
-        let grad =
-            colorgrad::GimpGradient::new(BufReader::new(GGR_SAMPLE.as_bytes()), &color, &color)
-                .unwrap();
-        self.display_gradient(Box::new(grad))?;
+        let br = BufReader::new(GGR_SAMPLE.as_bytes());
+        let grad = GimpGradient::new(br, &color, &color).unwrap();
+        self.display_gradient(&grad)?;
 
         writeln!(self.stdout, "{prompt} gradient --preset viridis --take 10")?;
         self.opt.custom = None;
-        self.opt.preset = Some("viridis".to_string());
+        self.opt.preset = Some("viridis".into());
         self.opt.take = Some(10);
         self.output_mode = OutputMode::ColorsN;
         self.preset_gradient()?;
